@@ -3,7 +3,7 @@
  *
  * Copyright (C) 2010-2012 SoftPLC Corporation, Dick Hollenbeck <dick@softplc.com>
  * Copyright (C) 2012 Wayne Stambaugh <stambaughw@gmail.com>
- * Copyright (C) 2012-2023 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -40,15 +40,13 @@
 
 #include <wx/dir.h>
 #include <wx/hash.h>
+#include <locale_io.h>
 
 #define OPT_SEP     '|'         ///< options separator character
 
 using namespace LIB_TABLE_T;
 
 
-static const wxChar global_tbl_name[] = wxT( "fp-lib-table" );
-
-
 bool FP_LIB_TABLE_ROW::operator==( const FP_LIB_TABLE_ROW& aRow ) const
 {
     return LIB_TABLE_ROW::operator == ( aRow ) && type == aRow.type;
@@ -62,7 +60,16 @@ void FP_LIB_TABLE_ROW::SetType( const wxString& aType )
     if( PCB_IO_MGR::PCB_FILE_T( -1 ) == type )
         type = PCB_IO_MGR::KICAD_SEXP;
 
-    plugin.release();
+    plugin.reset();
+}
+
+
+bool FP_LIB_TABLE_ROW::LibraryExists() const
+{
+    if( plugin )
+        return plugin->CanReadLibrary( GetFullURI( true ) );
+
+    return false;
 }
 
 
@@ -80,7 +87,7 @@ void FP_LIB_TABLE::Parse( LIB_TABLE_LEXER* in )
     wxString errMsg;    // to collect error messages
 
     // This table may be nested within a larger s-expression, or not.
-    // Allow for parser of that optional containing s-epression to have looked ahead.
+    // Allow for parser of that optional containing s-expression to have looked ahead.
     if( in->CurTok() != T_fp_lib_table )
     {
         in->NeedLEFT();
@@ -150,6 +157,7 @@ void FP_LIB_TABLE::Parse( LIB_TABLE_LEXER* in )
             case T_uri:
                 if( sawUri )
                     in->Duplicate( tok );
+
                 sawUri = true;
                 in->NeedSYMBOLorNUMBER();
                 row->SetFullURI( in->FromUTF8() );
@@ -158,6 +166,7 @@ void FP_LIB_TABLE::Parse( LIB_TABLE_LEXER* in )
             case T_type:
                 if( sawType )
                     in->Duplicate( tok );
+
                 sawType = true;
                 in->NeedSYMBOLorNUMBER();
                 row->SetType( in->FromUTF8() );
@@ -166,6 +175,7 @@ void FP_LIB_TABLE::Parse( LIB_TABLE_LEXER* in )
             case T_options:
                 if( sawOpts )
                     in->Duplicate( tok );
+
                 sawOpts = true;
                 in->NeedSYMBOLorNUMBER();
                 row->SetOptions( in->FromUTF8() );
@@ -174,6 +184,7 @@ void FP_LIB_TABLE::Parse( LIB_TABLE_LEXER* in )
             case T_descr:
                 if( sawDesc )
                     in->Duplicate( tok );
+
                 sawDesc = true;
                 in->NeedSYMBOLorNUMBER();
                 row->SetDescr( in->FromUTF8() );
@@ -182,6 +193,7 @@ void FP_LIB_TABLE::Parse( LIB_TABLE_LEXER* in )
             case T_disabled:
                 if( sawDisabled )
                     in->Duplicate( tok );
+
                 sawDisabled = true;
                 row->SetEnabled( false );
                 break;
@@ -205,15 +217,15 @@ void FP_LIB_TABLE::Parse( LIB_TABLE_LEXER* in )
         if( !sawUri )
             in->Expecting( T_uri );
 
-        // all nickNames within this table fragment must be unique, so we do not
-        // use doReplace in InsertRow().  (However a fallBack table can have a
-        // conflicting nickName and ours will supercede that one since in
-        // FindLib() we search this table before any fall back.)
-        wxString nickname = row->GetNickName(); // store it to be able to used it
-                                                // after row deletion if an error occurs
+        // All nickNames within this table fragment must be unique, so we do not use doReplace
+        // in doInsertRow().  (However a fallBack table can have a conflicting nickName and ours
+        // will supersede that one since in FindLib() we search this table before any fall back.)
+        wxString       nickname = row->GetNickName();   // store it to be able to used it
+                                                        // after row deletion if an error occurs
+        bool           doReplace = false;
         LIB_TABLE_ROW* tmp = row.release();
 
-        if( !InsertRow( tmp ) )
+        if( !doInsertRow( tmp, doReplace ) )
         {
             delete tmp;     // The table did not take ownership of the row.
 
@@ -301,40 +313,40 @@ long long FP_LIB_TABLE::GenerateTimestamp( const wxString* aNickname )
 
 
 void FP_LIB_TABLE::FootprintEnumerate( wxArrayString& aFootprintNames, const wxString& aNickname,
-                                       bool aBestEfforts )
+                                       bool aBestEfforts, const LOCALE_IO* aLocale )
 {
     const FP_LIB_TABLE_ROW* row = FindRow( aNickname, true );
     wxASSERT( row->plugin );
-    row->plugin->FootprintEnumerate( aFootprintNames, row->GetFullURI( true ), aBestEfforts,
-                                     row->GetProperties() );
-}
 
+    if( !aLocale )
+    {
+        LOCALE_IO toggle_locale;
 
-void FP_LIB_TABLE::PrefetchLib( const wxString& aNickname )
-{
-    const FP_LIB_TABLE_ROW* row = FindRow( aNickname, true );
-    wxASSERT( row->plugin );
-    row->plugin->PrefetchLib( row->GetFullURI( true ), row->GetProperties() );
+        row->plugin->FootprintEnumerate( aFootprintNames, row->GetFullURI( true ), aBestEfforts,
+                                         row->GetProperties() );
+    }
+    else
+    {
+        row->plugin->FootprintEnumerate( aFootprintNames, row->GetFullURI( true ), aBestEfforts,
+                                         row->GetProperties() );
+    }
 }
 
 
 const FP_LIB_TABLE_ROW* FP_LIB_TABLE::FindRow( const wxString& aNickname, bool aCheckIfEnabled )
 {
-    // Do not optimize this code.  Is done this way specifically to fix a runtime
-    // error with clang 4.0.1.
-    LIB_TABLE_ROW* ltrow = findRow( aNickname, aCheckIfEnabled );
-    FP_LIB_TABLE_ROW* row = dynamic_cast< FP_LIB_TABLE_ROW* >( ltrow );
+    FP_LIB_TABLE_ROW* row = static_cast<FP_LIB_TABLE_ROW*>( findRow( aNickname, aCheckIfEnabled ) );
 
     if( !row )
     {
-        wxString msg = wxString::Format( _( "fp-lib-table files contain no library named '%s'." ),
-                                         aNickname );
+        // We don't generally show this string to the user (who is unlikely to know what
+        // "fp-lib-table" means), and translating it may produce Sentry KICAD-YP.
+        wxString msg = wxString::Format( wxS( "'%s' not found in fp-lib-table." ), aNickname );
         THROW_IO_ERROR( msg );
     }
 
-    // We've been 'lazy' up until now, but it cannot be deferred any longer,
-    // instantiate a PCB_IO of the proper kind if it is not already in this
-    // FP_LIB_TABLE_ROW.
+    // We've been 'lazy' up until now, but it cannot be deferred any longer; instantiate a
+    // PCB_IO of the proper kind if it is not already in this FP_LIB_TABLE_ROW.
     if( !row->plugin )
         row->setPlugin( PCB_IO_MGR::PluginFind( row->type ) );
 
@@ -367,18 +379,33 @@ static void setLibNickname( FOOTPRINT* aModule, const wxString& aNickname,
 
 
 const FOOTPRINT* FP_LIB_TABLE::GetEnumeratedFootprint( const wxString& aNickname,
-                                                       const wxString& aFootprintName )
+                                                       const wxString& aFootprintName,
+                                                       const LOCALE_IO* aLocale )
 {
     const FP_LIB_TABLE_ROW* row = FindRow( aNickname, true );
     wxASSERT( row->plugin );
 
-    return row->plugin->GetEnumeratedFootprint( row->GetFullURI( true ), aFootprintName,
-                                                row->GetProperties() );
+    if( !aLocale )
+    {
+        LOCALE_IO toggle_locale;
+
+        return row->plugin->GetEnumeratedFootprint( row->GetFullURI( true ), aFootprintName,
+                                                    row->GetProperties() );
+    }
+    else
+    {
+        return row->plugin->GetEnumeratedFootprint( row->GetFullURI( true ), aFootprintName,
+                                                    row->GetProperties() );
+    }
 }
 
 
 bool FP_LIB_TABLE::FootprintExists( const wxString& aNickname, const wxString& aFootprintName )
 {
+    // NOT THREAD-SAFE!  LOCALE_IO is global!
+
+    LOCALE_IO toggle_locale;
+
     try
     {
         const FP_LIB_TABLE_ROW* row = FindRow( aNickname, true );
@@ -397,6 +424,10 @@ bool FP_LIB_TABLE::FootprintExists( const wxString& aNickname, const wxString& a
 FOOTPRINT* FP_LIB_TABLE::FootprintLoad( const wxString& aNickname,
                                         const wxString& aFootprintName, bool aKeepUUID )
 {
+    // NOT THREAD-SAFE!  LOCALE_IO is global!
+
+    LOCALE_IO toggle_locale;
+
     const FP_LIB_TABLE_ROW* row = FindRow( aNickname, true );
     wxASSERT( row->plugin );
 
@@ -412,6 +443,10 @@ FOOTPRINT* FP_LIB_TABLE::FootprintLoad( const wxString& aNickname,
 FP_LIB_TABLE::SAVE_T FP_LIB_TABLE::FootprintSave( const wxString& aNickname,
                                                   const FOOTPRINT* aFootprint, bool aOverwrite )
 {
+    // NOT THREAD-SAFE!  LOCALE_IO is global!
+
+    LOCALE_IO toggle_locale;
+
     const FP_LIB_TABLE_ROW* row = FindRow( aNickname, true );
     wxASSERT( row->plugin );
 
@@ -447,9 +482,21 @@ void FP_LIB_TABLE::FootprintDelete( const wxString& aNickname, const wxString& a
 
 bool FP_LIB_TABLE::IsFootprintLibWritable( const wxString& aNickname )
 {
-    const FP_LIB_TABLE_ROW* row = FindRow( aNickname, true );
-    wxASSERT( row->plugin );
-    return row->plugin->IsLibraryWritable( row->GetFullURI( true ) );
+    try
+    {
+        const FP_LIB_TABLE_ROW* row = FindRow( aNickname, true );
+
+        if( !row || !row->plugin )
+            return false;
+
+        return row->plugin->IsLibraryWritable( row->GetFullURI( true ) );
+    }
+    catch( ... )
+    {
+    }
+
+    // aNickname not found, so the library is not writable
+    return false;
 }
 
 
@@ -530,8 +577,8 @@ public:
         // it is under $KICADn_3RD_PARTY/footprints/<pkgid>/ i.e. has nested level of at least +3
         if( dirPath.EndsWith( wxS( ".pretty" ) ) && dir.GetDirCount() >= m_prefix_dir_count + 3 )
         {
-            wxString versionedPath = wxString::Format( wxS( "${%s}" ),
-                                       ENV_VAR::GetVersionedEnvVarName( wxS( "3RD_PARTY" ) ) );
+            wxString versionedPath = wxString::Format(
+                    wxS( "${%s}" ), ENV_VAR::GetVersionedEnvVarName( wxS( "3RD_PARTY" ) ) );
 
             wxArrayString parts = dir.GetDirs();
             parts.RemoveAt( 0, m_prefix_dir_count );
@@ -549,7 +596,8 @@ public:
                     int increment = 1;
                     do
                     {
-                        nickname = wxString::Format( wxS( "%s%s_%d" ), m_lib_prefix, name, increment );
+                        nickname = wxString::Format( wxS( "%s%s_%d" ), m_lib_prefix, name,
+                                                     increment );
                         increment++;
                     } while( m_lib_table.HasLibrary( nickname ) );
                 }
@@ -580,7 +628,8 @@ bool FP_LIB_TABLE::LoadGlobalTable( FP_LIB_TABLE& aTable )
     {
         tableExists = false;
 
-        if( !fn.DirExists() && !fn.Mkdir( 0x777, wxPATH_MKDIR_FULL ) )
+        if( !wxFileName::DirExists( fn.GetPath() )
+            && !wxFileName::Mkdir( fn.GetPath(), 0x777, wxPATH_MKDIR_FULL ) )
         {
             THROW_IO_ERROR( wxString::Format( _( "Cannot create global library table path '%s'." ),
                                               fn.GetPath() ) );
@@ -599,7 +648,7 @@ bool FP_LIB_TABLE::LoadGlobalTable( FP_LIB_TABLE& aTable )
         if( v && !v->IsEmpty() )
             ss.AddPaths( *v, 0 );
 
-        wxString fileName = ss.FindValidPath( global_tbl_name );
+        wxString fileName = ss.FindValidPath( FILEEXT::FootprintLibraryTableFileName );
 
         // The fallback is to create an empty global footprint table for the user to populate.
         if( fileName.IsEmpty() || !::wxCopyFile( fileName, fn.GetFullPath(), false ) )
@@ -610,11 +659,10 @@ bool FP_LIB_TABLE::LoadGlobalTable( FP_LIB_TABLE& aTable )
         }
     }
 
-    aTable.Clear();
     aTable.Load( fn.GetFullPath() );
 
     SETTINGS_MANAGER& mgr = Pgm().GetSettingsManager();
-    KICAD_SETTINGS*   settings = mgr.GetAppSettings<KICAD_SETTINGS>();
+    KICAD_SETTINGS*   settings = mgr.GetAppSettings<KICAD_SETTINGS>( "kicad" );
 
     const ENV_VAR_MAP& env = Pgm().GetLocalEnvVariables();
     wxString packagesPath;
@@ -665,7 +713,7 @@ wxString FP_LIB_TABLE::GetGlobalTableFileName()
     wxFileName fn;
 
     fn.SetPath( PATHS::GetUserSettingsPath() );
-    fn.SetName( global_tbl_name );
+    fn.SetName( FILEEXT::FootprintLibraryTableFileName );
 
     return fn.GetFullPath();
 }
