@@ -284,6 +284,12 @@ wxString FILEEXT::KiCadSchematicFileWildcard()
 }
 
 
+wxString FILEEXT::AltiumProjectFilesWildcard()
+{
+    return _( "Altium Project files" ) + AddFileExtListToFilter( { "PrjPcb" } );
+}
+
+
 wxString FILEEXT::CadstarArchiveFilesWildcard()
 {
     return _( "CADSTAR Archive files" ) + AddFileExtListToFilter( { "csa", "cpa" } );
