@@ -84,7 +84,8 @@ BOARD::BOARD() :
         m_userUnits( EDA_UNITS::MM ),
         m_designSettings( new BOARD_DESIGN_SETTINGS( nullptr, "board.design_settings" ) ),
         m_NetInfo( this ),
-        m_embedFonts( false )
+        m_embedFonts( false ),
+        m_embeddedFilesDelegate( nullptr )
 {
     // A too small value do not allow connecting 2 shapes (i.e. segments) not exactly connected
     // A too large value do not allow safely connecting 2 shapes like very short segments.
@@ -257,6 +258,8 @@ void BOARD::ClearProject()
 
 void BOARD::IncrementTimeStamp()
 {
+    std::unique_lock<std::shared_mutex> writeLock( m_CachesMutex );
+
     m_timeStamp++;
 
     if( !m_IntersectsAreaCache.empty()
@@ -267,10 +270,9 @@ void BOARD::IncrementTimeStamp()
         || !m_LayerExpressionCache.empty()
         || !m_ZoneBBoxCache.empty()
         || m_CopperItemRTreeCache
-        || m_maxClearanceValue.has_value() )
+        || m_maxClearanceValue.has_value()
+        || !m_itemByIdCache.empty() )
     {
-        std::unique_lock<std::shared_mutex> writeLock( m_CachesMutex );
-
         m_IntersectsAreaCache.clear();
         m_EnclosedByAreaCache.clear();
         m_IntersectsCourtyardCache.clear();
@@ -292,6 +294,8 @@ void BOARD::IncrementTimeStamp()
         m_CopperZoneRTreeCache.clear();
 
         m_maxClearanceValue.reset();
+
+        m_itemByIdCache.clear();
     }
 }
 
@@ -341,8 +345,49 @@ void BOARD::RecordDRCExclusions()
             m_designSettings->m_DrcExclusionComments[ serialized ] = marker->GetComment();
         }
     }
+
+    if( m_project )
+    {
+        if( PROJECT_FILE* projectFile = &m_project->GetProjectFile() )
+        {
+            if( BOARD_DESIGN_SETTINGS* prjSettings = projectFile->m_BoardSettings )
+            {
+                prjSettings->m_DrcExclusions = m_designSettings->m_DrcExclusions;
+                prjSettings->m_DrcExclusionComments = m_designSettings->m_DrcExclusionComments;
+            }
+        }
+    }
 }
 
+std::set<wxString>::iterator FindByFirstNFields( std::set<wxString>& strSet,
+                                                 const wxString& searchStr, char delimiter, int n )
+{
+    wxString searchPrefix = searchStr;
+
+    // Extract first n fields from the search string
+    int    delimiterCount = 0;
+    size_t pos = 0;
+
+    while( pos < searchPrefix.length() && delimiterCount < n )
+    {
+        if( searchPrefix[pos] == delimiter )
+            delimiterCount++;
+        pos++;
+    }
+
+    if( delimiterCount == n )
+        searchPrefix = searchPrefix.Left( pos - 1 ); // Exclude the nth delimiter
+
+    for( auto it = strSet.begin(); it != strSet.end(); ++it )
+    {
+        if( it->StartsWith( searchPrefix + delimiter ) || *it == searchPrefix )
+        {
+            return it;
+        }
+    }
+
+    return strSet.end();
+}
 
 std::vector<PCB_MARKER*> BOARD::ResolveDRCExclusions( bool aCreateMarkers )
 {
@@ -354,16 +399,39 @@ std::vector<PCB_MARKER*> BOARD::ResolveDRCExclusions( bool aCreateMarkers )
 
     for( PCB_MARKER* marker : GetBoard()->Markers() )
     {
+        std::set<wxString>::iterator it;
         wxString                     serialized = marker->SerializeToString();
-        std::set<wxString>::iterator it = exclusions.find( serialized );
+        wxString                     matchedExclusion;
 
+        if( !serialized.Contains( "unconnected_items" ) )
+        {
+            it = exclusions.find( serialized );
+            if( it != exclusions.end() )
+            {
+                matchedExclusion = *it;
+            }
+        }
+        else
+        {
+            const int  numberOfFieldsExcludingIds = 3;
+            const char delimiter = '|';
+            it = FindByFirstNFields( exclusions, serialized, delimiter,
+                                     numberOfFieldsExcludingIds );
+            if( it != exclusions.end() )
+            {
+                matchedExclusion = *it;
+            }
+            else
+            {
+            }
+        }
         if( it != exclusions.end() )
         {
-            marker->SetExcluded( true, comments[ serialized ] );
+            marker->SetExcluded( true, comments[matchedExclusion] );
 
             // Exclusion still valid; store back to BOARD_DESIGN_SETTINGS
-            m_designSettings->m_DrcExclusions.insert( serialized );
-            m_designSettings->m_DrcExclusionComments[ serialized ] = comments[ serialized ];
+            m_designSettings->m_DrcExclusions.insert( matchedExclusion );
+            m_designSettings->m_DrcExclusionComments[matchedExclusion] = comments[matchedExclusion];
 
             exclusions.erase( it );
         }
@@ -648,6 +716,18 @@ bool BOARD::SetLayerName( PCB_LAYER_ID aLayer, const wxString& aLayerName )
 }
 
 
+bool BOARD::IsFrontLayer( PCB_LAYER_ID aLayer ) const
+{
+    return ::IsFrontLayer( aLayer ) || GetLayerType( aLayer ) == LT_FRONT;
+}
+
+
+bool BOARD::IsBackLayer( PCB_LAYER_ID aLayer ) const
+{
+    return ::IsBackLayer( aLayer ) || GetLayerType( aLayer ) == LT_BACK;
+}
+
+
 LAYER_T BOARD::GetLayerType( PCB_LAYER_ID aLayer ) const
 {
     if( IsLayerEnabled( aLayer ) )
@@ -825,7 +905,7 @@ int BOARD::LayerDepth( PCB_LAYER_ID aStartLayer, PCB_LAYER_ID aEndLayer ) const
 }
 
 
-LSET BOARD::GetEnabledLayers() const
+const LSET& BOARD::GetEnabledLayers() const
 {
     return GetDesignSettings().GetEnabledLayers();
 }
@@ -839,13 +919,13 @@ bool BOARD::IsLayerVisible( PCB_LAYER_ID aLayer ) const
 }
 
 
-LSET BOARD::GetVisibleLayers() const
+const LSET& BOARD::GetVisibleLayers() const
 {
     return m_project ? m_project->GetLocalSettings().m_VisibleLayers : LSET::AllLayersMask();
 }
 
 
-void BOARD::SetEnabledLayers( LSET aLayerSet )
+void BOARD::SetEnabledLayers( const LSET& aLayerSet )
 {
     GetDesignSettings().SetEnabledLayers( aLayerSet );
 }
@@ -857,7 +937,7 @@ bool BOARD::IsLayerEnabled( PCB_LAYER_ID aLayer ) const
 }
 
 
-void BOARD::SetVisibleLayers( LSET aLayerSet )
+void BOARD::SetVisibleLayers( const LSET& aLayerSet )
 {
     if( m_project )
         m_project->GetLocalSettings().m_VisibleLayers = aLayerSet;
@@ -887,7 +967,7 @@ void BOARD::SetVisibleAlls()
 
 GAL_SET BOARD::GetVisibleElements() const
 {
-    return m_project ? m_project->GetLocalSettings().m_VisibleItems : GAL_SET().set();
+    return m_project ? m_project->GetLocalSettings().m_VisibleItems : GAL_SET::DefaultVisible();
 }
 
 
@@ -1030,23 +1110,31 @@ void BOARD::CacheTriangulation( PROGRESS_REPORTER* aReporter, const std::vector<
 }
 
 
-void BOARD::FixupEmbeddedData()
+void BOARD::RunOnNestedEmbeddedFiles( const std::function<void( EMBEDDED_FILES* )>& aFunction )
 {
     for( FOOTPRINT* footprint : m_footprints )
-    {
-        for( auto& [filename, embeddedFile] : footprint->EmbeddedFileMap() )
-        {
-            EMBEDDED_FILES::EMBEDDED_FILE* file = GetEmbeddedFile( filename );
+        aFunction( footprint->GetEmbeddedFiles() );
+}
 
-            if( file )
+
+void BOARD::FixupEmbeddedData()
+{
+    RunOnNestedEmbeddedFiles(
+            [&]( EMBEDDED_FILES* nested )
             {
-                embeddedFile->compressedEncodedData = file->compressedEncodedData;
-                embeddedFile->decompressedData = file->decompressedData;
-                embeddedFile->data_hash = file->data_hash;
-                embeddedFile->is_valid = file->is_valid;
-            }
-        }
-    }
+                for( auto& [filename, embeddedFile] : nested->EmbeddedFileMap() )
+                {
+                    EMBEDDED_FILES::EMBEDDED_FILE* file = GetEmbeddedFile( filename );
+
+                    if( file )
+                    {
+                        embeddedFile->compressedEncodedData = file->compressedEncodedData;
+                        embeddedFile->decompressedData = file->decompressedData;
+                        embeddedFile->data_hash = file->data_hash;
+                        embeddedFile->is_valid = file->is_valid;
+                    }
+                }
+            } );
 }
 
 
@@ -1086,14 +1174,24 @@ void BOARD::Add( BOARD_ITEM* aBoardItem, ADD_MODE aMode, bool aSkipConnectivity
         m_zones.push_back( (ZONE*) aBoardItem );
         break;
 
-    case PCB_TRACE_T:
     case PCB_VIA_T:
-    case PCB_ARC_T:
+        if( aMode == ADD_MODE::APPEND || aMode == ADD_MODE::BULK_APPEND )
+            m_tracks.push_back( static_cast<PCB_VIA*>( aBoardItem ) );
+        else
+            m_tracks.push_front( static_cast<PCB_VIA*>( aBoardItem ) );
 
-        // N.B. This inserts a small memory leak as we lose the
+        break;
+
+
+    case PCB_TRACE_T:
+    case PCB_ARC_T:
         if( !IsCopperLayer( aBoardItem->GetLayer() ) )
         {
-            wxFAIL_MSG( wxT( "BOARD::Add() Cannot place Track on non-copper layer" ) );
+            // The only current known source of these is SWIG (KICAD-BY7, et al).
+            // N.B. This inserts a small memory leak as we lose the track/via/arc.
+            wxFAIL_MSG( wxString::Format( "BOARD::Add() Cannot place Track on non-copper layer: %d = %s",
+                                          static_cast<int>( aBoardItem->GetLayer() ),
+                                          GetLayerName( aBoardItem->GetLayer() ) ) );
             return;
         }
 
@@ -1386,13 +1484,138 @@ void BOARD::RemoveAll( std::initializer_list<KICAD_T> aTypes )
         }
     }
 
-    for( BOARD_ITEM* item : removed )
-        m_itemByIdCache.erase( item->m_Uuid );
+    IncrementTimeStamp();
 
     FinalizeBulkRemove( removed );
 }
 
 
+bool BOARD::HasItemsOnLayer( PCB_LAYER_ID aLayer )
+{
+    PCB_LAYER_COLLECTOR collector;
+
+    collector.SetLayerId( aLayer );
+    collector.Collect( this, GENERAL_COLLECTOR::BoardLevelItems );
+
+    if( collector.GetCount() != 0 )
+    {
+        // Skip items owned by footprints and footprints when building
+        // the actual list of removed layers: these items are not removed
+        for( int i = 0; i < collector.GetCount(); i++ )
+        {
+            BOARD_ITEM* item = collector[i];
+
+            if( item->Type() == PCB_FOOTPRINT_T || item->GetParentFootprint() )
+                continue;
+
+            // Vias are on multiple adjacent layers, but only the top and
+            // the bottom layers are stored. So there are issues only if one
+            // is on a removed layer
+            if( item->Type() == PCB_VIA_T )
+            {
+                PCB_VIA* via = static_cast<PCB_VIA*>( item );
+
+                if( via->GetViaType() == VIATYPE::THROUGH )
+                    continue;
+                else
+                {
+                    PCB_LAYER_ID top_layer;
+                    PCB_LAYER_ID bottom_layer;
+                    via->LayerPair( &top_layer, &bottom_layer );
+
+                    if( top_layer != aLayer && bottom_layer != aLayer )
+                        continue;
+                }
+            }
+
+            return true;
+        }
+    }
+
+    return false;
+}
+
+
+bool BOARD::RemoveAllItemsOnLayer( PCB_LAYER_ID aLayer )
+{
+    bool modified = false;
+    bool removedItemLayers = false;
+    PCB_LAYER_COLLECTOR collector;
+
+    collector.SetLayerId( aLayer );
+    collector.Collect( this, GENERAL_COLLECTOR::BoardLevelItems );
+
+    for( int i = 0; i < collector.GetCount(); i++ )
+    {
+        BOARD_ITEM* item = collector[i];
+
+        // Do not remove/change an item owned by a footprint
+        if( item->GetParentFootprint() )
+            continue;
+
+        // Do not remove footprints
+        if( item->Type() == PCB_FOOTPRINT_T )
+            continue;
+
+        // Note: vias are specific. They are only on copper layers,  and
+        // do not use a layer set, only store the copper top and the copper bottom.
+        // So reinit the layer set does not work with vias
+        if( item->Type() == PCB_VIA_T )
+        {
+            PCB_VIA* via = static_cast<PCB_VIA*>( item );
+
+            if( via->GetViaType() == VIATYPE::THROUGH )
+            {
+                removedItemLayers = true;
+                continue;
+            }
+            else if( via->IsOnLayer( aLayer ) )
+            {
+                PCB_LAYER_ID top_layer;
+                PCB_LAYER_ID bottom_layer;
+                via->LayerPair( &top_layer, &bottom_layer );
+
+                if( top_layer == aLayer || bottom_layer == aLayer )
+                {
+                    // blind/buried vias with a top or bottom layer on a removed layer
+                    // are removed. Perhaps one could just modify the top/bottom layer,
+                    // but I am not sure this is better.
+                    Remove( item );
+                    delete item;
+                    modified = true;
+                }
+
+                removedItemLayers = true;
+            }
+        }
+        else if( item->IsOnLayer( aLayer ) )
+        {
+            LSET layers = item->GetLayerSet();
+
+            layers.reset( aLayer );
+
+            if( layers.any() )
+            {
+                item->SetLayerSet( layers );
+            }
+            else
+            {
+                Remove( item );
+                delete item;
+                modified = true;
+            }
+
+            removedItemLayers = true;
+        }
+    }
+
+    if( removedItemLayers )
+        BuildConnectivity();
+
+    return modified;
+}
+
+
 wxString BOARD::GetItemDescription( UNITS_PROVIDER* aUnitsProvider, bool aFull ) const
 {
     return wxString::Format( _( "PCB" ) );
@@ -1427,30 +1650,24 @@ void BOARD::UpdateUserUnits( BOARD_ITEM* aItem, KIGFX::VIEW* aView )
 
 void BOARD::DeleteMARKERs()
 {
-    // the vector does not know how to delete the PCB_MARKER, it holds pointers
     for( PCB_MARKER* marker : m_markers )
-    {
-        // We also must clear the cache
-        m_itemByIdCache.erase( marker->m_Uuid );
         delete marker;
-    }
 
     m_markers.clear();
+    IncrementTimeStamp();
 }
 
 
 void BOARD::DeleteMARKERs( bool aWarningsAndErrors, bool aExclusions )
 {
     // Deleting lots of items from a vector can be very slow.  Copy remaining items instead.
-    MARKERS remaining;
+    std::vector<PCB_MARKER*> remaining;
 
     for( PCB_MARKER* marker : m_markers )
     {
         if( ( marker->GetSeverity() == RPT_SEVERITY_EXCLUSION && aExclusions )
                 || ( marker->GetSeverity() != RPT_SEVERITY_EXCLUSION && aWarningsAndErrors ) )
         {
-            // We also must clear the cache
-            m_itemByIdCache.erase( marker->m_Uuid );
             delete marker;
         }
         else
@@ -1459,7 +1676,8 @@ void BOARD::DeleteMARKERs( bool aWarningsAndErrors, bool aExclusions )
         }
     }
 
-    m_markers = remaining;
+    m_markers = std::move( remaining );
+    IncrementTimeStamp();
 }
 
 
@@ -1467,7 +1685,6 @@ void BOARD::DeleteAllFootprints()
 {
     for( FOOTPRINT* footprint : m_footprints )
     {
-        m_itemByIdCache.erase( footprint->m_Uuid );
         delete footprint;
     }
 
@@ -1476,6 +1693,18 @@ void BOARD::DeleteAllFootprints()
 }
 
 
+void BOARD::DetachAllFootprints()
+{
+    for( FOOTPRINT* footprint : m_footprints )
+    {
+        footprint->SetParent( nullptr );
+    }
+
+    m_footprints.clear();
+    IncrementTimeStamp();
+}
+
+
 BOARD_ITEM* BOARD::GetItem( const KIID& aID ) const
 {
     if( aID == niluuid )
@@ -1992,8 +2221,6 @@ NETINFO_ITEM* BOARD::FindNet( int aNetcode ) const
     // zero is reserved for "no connection" and is not actually a net.
     // nullptr is returned for non valid netcodes
 
-    wxASSERT( m_NetInfo.GetNetCount() > 0 );
-
     if( aNetcode == NETINFO_LIST::UNCONNECTED && m_NetInfo.GetNetCount() == 0 )
         return NETINFO_LIST::OrphanedItem();
     else
@@ -2112,9 +2339,7 @@ std::set<wxString> BOARD::GetNetClassAssignmentCandidates() const
 
 void BOARD::SynchronizeProperties()
 {
-    wxCHECK( m_project, /*void*/ );
-
-    if( !m_project->IsNullProject() )
+    if( m_project && !m_project->IsNullProject() )
         SetProperties( m_project->GetTextVars() );
 }
 
@@ -2213,25 +2438,6 @@ PAD* BOARD::GetPad( const PCB_TRACK* aTrace, ENDPOINT_T aEndPoint ) const
 }
 
 
-PAD* BOARD::GetPadFast( const VECTOR2I& aPosition, LSET aLayerSet ) const
-{
-    for( FOOTPRINT* footprint : Footprints() )
-    {
-        for( PAD* pad : footprint->Pads() )
-        {
-        if( pad->GetPosition() != aPosition )
-            continue;
-
-        // Pad found, it must be on the correct layer
-        if( ( pad->GetLayerSet() & aLayerSet ).any() )
-            return pad;
-        }
-    }
-
-    return nullptr;
-}
-
-
 PAD* BOARD::GetPad( std::vector<PAD*>& aPadList, const VECTOR2I& aPosition, LSET aLayerSet ) const
 {
     // Search aPadList for aPosition
@@ -2536,31 +2742,6 @@ std::list<ZONE*> BOARD::GetZoneList( bool aIncludeZonesInFootprints ) const
 }
 
 
-ZONE* BOARD::AddArea( PICKED_ITEMS_LIST* aNewZonesList, int aNetcode, PCB_LAYER_ID aLayer,
-                      VECTOR2I aStartPointPosition, ZONE_BORDER_DISPLAY_STYLE aHatch )
-{
-    ZONE* new_area = new ZONE( this );
-
-    new_area->SetNetCode( aNetcode );
-    new_area->SetLayer( aLayer );
-
-    m_zones.push_back( new_area );
-
-    new_area->SetHatchStyle( (ZONE_BORDER_DISPLAY_STYLE) aHatch );
-
-    // Add the first corner to the new zone
-    new_area->AppendCorner( aStartPointPosition, -1 );
-
-    if( aNewZonesList )
-    {
-        ITEM_PICKER picker( nullptr, new_area, UNDO_REDO::NEWITEM );
-        aNewZonesList->PushItem( picker );
-    }
-
-    return new_area;
-}
-
-
 bool BOARD::GetBoardPolygonOutlines( SHAPE_POLY_SET& aOutlines,
                                      OUTLINE_ERROR_HANDLER* aErrorHandler,
                                      bool aAllowUseArcsInPolygons,
@@ -2616,8 +2797,8 @@ bool BOARD::GetBoardPolygonOutlines( SHAPE_POLY_SET& aOutlines,
 
 EMBEDDED_FILES* BOARD::GetEmbeddedFiles()
 {
-    if( IsFootprintHolder() )
-        return static_cast<EMBEDDED_FILES*>( GetFirstFootprint() );
+    if( m_embeddedFilesDelegate )
+        return static_cast<EMBEDDED_FILES*>( m_embeddedFilesDelegate );
 
     return static_cast<EMBEDDED_FILES*>( this );
 }
@@ -2625,8 +2806,8 @@ EMBEDDED_FILES* BOARD::GetEmbeddedFiles()
 
 const EMBEDDED_FILES* BOARD::GetEmbeddedFiles() const
 {
-    if( IsFootprintHolder() )
-        return static_cast<const EMBEDDED_FILES*>( GetFirstFootprint() );
+    if( m_embeddedFilesDelegate )
+        return static_cast<const EMBEDDED_FILES*>( m_embeddedFilesDelegate );
 
     return static_cast<const EMBEDDED_FILES*>( this );
 }
@@ -2634,6 +2815,8 @@ const EMBEDDED_FILES* BOARD::GetEmbeddedFiles() const
 
 std::set<KIFONT::OUTLINE_FONT*> BOARD::GetFonts() const
 {
+    using PERMISSION = KIFONT::OUTLINE_FONT::EMBEDDING_PERMISSION;
+
     std::set<KIFONT::OUTLINE_FONT*> fonts;
 
     for( BOARD_ITEM* item : Drawings() )
@@ -2642,16 +2825,13 @@ std::set<KIFONT::OUTLINE_FONT*> BOARD::GetFonts() const
         {
             KIFONT::FONT* font = text->GetFont();
 
-            if( !font || font->IsStroke() )
-                continue;
-
-            using EMBEDDING_PERMISSION = KIFONT::OUTLINE_FONT::EMBEDDING_PERMISSION;
-            auto* outline = static_cast<KIFONT::OUTLINE_FONT*>( font );
-
-            if( outline->GetEmbeddingPermission() == EMBEDDING_PERMISSION::EDITABLE
-                || outline->GetEmbeddingPermission() == EMBEDDING_PERMISSION::INSTALLABLE )
+            if( font && font->IsOutline() )
             {
-                fonts.insert( outline );
+                KIFONT::OUTLINE_FONT* outlineFont = static_cast<KIFONT::OUTLINE_FONT*>( font );
+                PERMISSION            permission = outlineFont->GetEmbeddingPermission();
+
+                if( permission == PERMISSION::EDITABLE || permission == PERMISSION::INSTALLABLE )
+                    fonts.insert( outlineFont );
             }
         }
     }
@@ -2662,9 +2842,7 @@ std::set<KIFONT::OUTLINE_FONT*> BOARD::GetFonts() const
 
 void BOARD::EmbedFonts()
 {
-    std::set<KIFONT::OUTLINE_FONT*> fonts = GetFonts();
-
-    for( KIFONT::OUTLINE_FONT* font : fonts )
+    for( KIFONT::OUTLINE_FONT* font : GetFonts() )
     {
         auto file = GetEmbeddedFiles()->AddFile( font->GetFileName(), false );
         file->type = EMBEDDED_FILES::EMBEDDED_FILE::FILE_TYPE::FONT;
